from crispy_forms.helper import FormHelper
from crispy_forms.layout import Submit
from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from django_summernote_ajax import settings as django_summernote_ajax_settings
from . import settings as sandbox_app_settings
from .models import Post
from .widgets import (
    PostSummernoteBs4Widget, PostAdminSummernoteLiteWidget
)


class PostForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(PostForm, self).__init__(*args, **kwargs)

        self.helper = FormHelper()
        self.helper.form_group_wrapper_class = 'row'
        self.helper.label_class = 'col-2 col-form-label'
        self.helper.field_class = 'col-10'
        self.helper.include_media = False
        self.helper.add_input(Submit('submit', 'Write'))

    class Meta:
        model = Post
        fields = ['title', 'body']
        widgets = {
            'body': PostSummernoteBs4Widget(
                options={
                    'upload_url': settings.POST_FILE_UPLOAD_URL,
                    'delete_url': settings.POST_FILE_DELETE_URL,
                    'max_file_count': settings.POST_MAX_FILE_COUNT,
                    'max_upload_size': django_summernote_ajax_settings.DSA_MAX_UPLOAD_SIZE,
                    'content_types': django_summernote_ajax_settings.DSA_CONTENT_TYPES,
                    'file_extensions': django_summernote_ajax_settings.DSA_FILE_EXTENSIONS,
                }
            ),
        }


class PostAttachmentForm(PostForm):
    attachments = forms.UUIDField(widget=forms.HiddenInput(), required=False)

    def clean_attachments(self):
        data = self.data.getlist('attachments')

        count = self.instance.attachments.count() if self.instance else 0

        if count + len(data) > sandbox_app_settings.POST_MAX_FILE_COUNT:
            raise forms.ValidationError(_('Maximum number of files exceeded.'))

        return data


class PostAdminForm(forms.ModelForm):
    class Meta:
        model = Post
        fields = '__all__'
        widgets = {
            'body': PostAdminSummernoteLiteWidget(wrapper_class='summernote-wrapper-fixed'),
        }



class AuthorRequiredMixin(object):
    """
    Authors may edit or delete their own posts only.
    """

    def get_queryset(self):
        queryset = super(AuthorRequiredMixin, self).get_queryset()
        return queryset.filter(author=self.request.user)



from django.contrib import admin
from django.contrib.admin.filters import SimpleListFilter

from . import settings as sandbox_app_settings
from .forms import PostAdminForm
from .models import (
    Attachment, Post
)


class NullFilterSpec(SimpleListFilter):
    title = ''
    parameter_name = ''

    def lookups(self, request, model_admin):
        return (
            ('1', 'Has post',),
            ('0', 'No post',),
        )

    def queryset(self, request, queryset):
        kwargs = {
            '%s' % self.parameter_name: None,
        }
        if self.value() == '0':
            return queryset.filter(**kwargs)
        if self.value() == '1':
            return queryset.exclude(**kwargs)
        return queryset


class PostNullFilterSpec(NullFilterSpec):
    title = 'post'
    parameter_name = 'post'


class AttachmentInline(admin.StackedInline):
    model = Attachment
    extra = 2
    max_num = sandbox_app_settings.POST_MAX_FILE_COUNT


class AttachmentAdmin(admin.ModelAdmin):
    list_display = ('name', 'post', 'file', 'created')
    list_filter = (PostNullFilterSpec,)
    ordering = ('-created',)


class PostAdmin(admin.ModelAdmin):
    list_display = ('title', 'author', 'created')
    form = PostAdminForm
    inlines = [AttachmentInline]
    ordering = ('-created',)

    class Media:
        js = (
            # django admin jQuery is not compatible with summernote.
            '//ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js',
        )


admin.site.register(Post, PostAdmin)
admin.site.register(Attachment, AttachmentAdmin)



from django.apps import apps
from django.contrib import auth
from django.contrib.auth.models import User
from django.test import (
    TestCase, Client
)
from django.utils import timezone

from .apps import SandboxAppConfig
from .forms import (
    PostForm, PostAttachmentForm, PostAdminForm
)
from .models import (
    Post, upload_directory_path
)


class SandboxAppTest(TestCase):
    def setUp(self):
        self.username = 'dsa'
        self.password = 'pass'

    @classmethod
    def setUpTestData(cls):
        Post.objects.create(title="post title", body="post body content", created=timezone.now())

    def tearDown(self):
        pass

    # apps
    def test_apps(self):
        self.assertEqual(SandboxAppConfig.name, 'sandbox_app')
        self.assertEqual(SandboxAppConfig.verbose_name, 'sandbox')
        self.assertEqual(apps.get_app_config('sandbox_app').name, 'sandbox_app')

    # models
    def test_post_creation(self):
        post = Post.objects.get(id=1)
        self.assertTrue(isinstance(post, Post))
        self.assertEqual(post.title, post.__str__())
        self.assertEqual(post.title, "post title")

    def test_post_get_absolute_url(self):
        post = Post.objects.get(id=1)
        self.assertEquals(post.get_absolute_url(), '/posts/1/')

    def test_upload_directory_path(self):
        path = upload_directory_path(None, "test.png")
        self.assertNotEqual("test.png", path)
        self.assertTrue(path.startswith('attachments'))
        self.assertTrue(path.endswith('png'))

    # widgets

    # forms
    def test_valid_post_form(self):
        post = Post.objects.get(id=1)
        form = PostForm(data={'title': post.title, 'body': post.body})
        html = form.as_p()
        self.assertIn('summernote-widget', html)
        self.assertTrue(form.is_valid())

    def test_invalid_post_form(self):
        form = PostForm(data={'title': '', 'body': ''})
        html = form.as_p()
        self.assertIn('summernote-widget', html)
        self.assertFalse(form.is_valid())

    def test_invalid_empty_post_attachment_form(self):
        form = PostAttachmentForm()
        html = form.as_p()
        self.assertIn('summernote-widget', html)
        self.assertFalse(form.is_valid())

    def test_valid_empty_post_attachment_form(self):
        # empty list of attachment
        pass

    def test_valid_post_attachment_form(self):
        pass

    def test_over_max_post_attachment_form(self):
        pass

    def test_valid_post_admin_form(self):
        post = Post.objects.get(id=1)
        form = PostAdminForm(data={'title': post.title, 'body': post.body})
        html = form.as_p()
        self.assertIn('summernote-widget', html)
        self.assertTrue(form.is_valid())

    def test_invalid_post_admin_form(self):
        form = PostAdminForm(data={'title': '', 'body': ''})
        html = form.as_p()
        self.assertIn('summernote-widget', html)
        self.assertFalse(form.is_valid())

    # viewmxins

    # views
    def test_user_is_not_authenticated(self):
        user = auth.get_user(self.client)
        self.assertTrue(not user.is_authenticated)

    def test_login(self):
        self.user = User.objects.create_user(username=self.username, password=self.password)
        self.assertTrue(Client().login(username=self.username, password=self.password))



# Generated by Django 2.0.1 on 2018-01-26 10:54

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import sandbox_app.models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=255, null=True, verbose_name='file name')),
                ('uid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True, verbose_name='public identifier')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created time')),
                ('file', models.FileField(upload_to=sandbox_app.models.upload_directory_path, verbose_name='uploaded file')),
            ],
            options={
                'verbose_name': 'attachment',
                'verbose_name_plural': 'attachments',
                'ordering': ['-created'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, verbose_name='title')),
                ('body', models.TextField(verbose_name='content body')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created time')),
                ('author', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='posts', to=settings.AUTH_USER_MODEL, verbose_name='author')),
            ],
            options={
                'ordering': ['-created'],
            },
        ),
        migrations.AddField(
            model_name='attachment',
            name='post',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='sandbox_app.Post', verbose_name='post'),
        ),
    ]



from celery import shared_task
from django.core.files.storage import default_storage

from .models import Attachment


@shared_task
def delete_orphan_attachments():
    attachments = Attachment.objects.filter(post__isnull=True)

    count = len(attachments)

    for attachment in attachments:
        default_storage.delete(attachment.file.path)

    attachments.delete()

    return count



import json

from django.forms import widgets
from django.template import loader
from django.utils.safestring import mark_safe


class SummernoteWidgetBase(widgets.Textarea):
    template_name = 'django_summernote_ajax/django_summernote_ajax.html'

    def __init__(self, attrs=None, wrapper_class='', options={}):
        self.wrapper_class = wrapper_class
        self.options = options

        super(SummernoteWidgetBase, self).__init__(attrs=attrs)

    def render(self, name, value, attrs=None, renderer=None):
        context = {
            'widget': {
                'name': name,
                'value': value,
                'wrapper_class': self.wrapper_class,
                'options': json.dumps(self.options),
            }
        }

        template = loader.get_template(self.template_name).render(context)
        return mark_safe(template)


class SummernoteLiteWidget(SummernoteWidgetBase):
    class Media:
        css = {
            'all': (
                'css/summernote/summernote-lite.css',
            )
        }
        js = (
            'js/summernote/csrf-cookie.js',
            'js/summernote/summernote-lite.js',
        )


class SummernoteBs3Widget(SummernoteWidgetBase):
    class Media:
        css = {
            'all': (
                'css/summernote/summernote.css',
            )
        }
        js = (
            'js/summernote/csrf-cookie.js',
            'js/summernote/summernote.min.js',
        )


class SummernoteBs4Widget(SummernoteWidgetBase):
    class Media:
        css = {
            'all': (
                'css/summernote/summernote-bs4.css',
            )
        }
        js = (
            'js/summernote/csrf-cookie.js',
            'js/summernote/summernote-bs4.min.js',
        )



from django_summernote_ajax.widgets import (
    SummernoteBs4Widget, SummernoteLiteWidget
)


class PostSummernoteBs4Widget(SummernoteBs4Widget):
    class Media:
        js = (
            'js/post-summernote-ajax.js',
        )


class PostAdminSummernoteLiteWidget(SummernoteLiteWidget):
    class Media:
        css = {
            'all': (
                'css/django-summernote.css',
            )
        }
        js = (
            'js/post-admin-summernote-ajax.js',
        )



from django.contrib.auth.mixins import LoginRequiredMixin
from django.urls import reverse_lazy
from django.views.generic import (
    ListView, DetailView
)
from django.views.generic.edit import (
    CreateView, UpdateView, DeleteView
)

from django_summernote_ajax.views import (
    FileUploadView, FileDeleteView
)
from .forms import (
    PostForm, PostAttachmentForm
)
from .models import (
    Attachment, Post
)
from .viewmixins import AuthorRequiredMixin


class PostAttachmentUploadView(LoginRequiredMixin, FileUploadView):
    # Raise PermissionDenied exception instead of the redirect
    raise_exception = True

    def upload_file(self, *args, **kwargs):
        """
        This method must be overridden to perform uploading files and return JSON file list.
        """
        uploaded_files = kwargs.pop('uploaded_files', None)
        files = []

        if uploaded_files:
            # NOTE: Change HTML attribute name - `files`
            for file in uploaded_files.getlist('files'):
                # NOTE: Attachment class must inherit AbstractAttachment
                attachment = Attachment()
                attachment.file = file
                attachment.name = file.name
                attachment.save()

                files.append({
                    "uid": attachment.uid,
                    "name": file.name,
                    "size": file.size,
                    "url": attachment.file.url
                })

        # NOTE: Change JSON key name - `files`
        return {"files": files}


class PostAttachmentDeleteView(LoginRequiredMixin, FileDeleteView):
    # Raise PermissionDenied exception instead of the redirect
    raise_exception = True

    def delete_file(self, *args, **kwargs):
        """
        This method must be overridden to perform deleting files and return JSON file list.
        """
        form = kwargs.pop('form', None)
        user = kwargs.pop('user', None)

        files = []

        if form and user:
            # NOTE: Attachment class must inherit AbstractAttachment and asynchronously deleted by cron.
            attachment = Attachment.objects.select_related('post__author').get(uid=form.cleaned_data['uid'])

            if attachment.post and attachment.post.author == user:
                attachment.post = None
                attachment.save()

            files.append({
                "uid": attachment.uid,
            })

        # NOTE: Change JSON key name - `files`
        return {"files": files}


class PostListView(ListView):
    model = Post
    context_object_name = 'posts'
    template_name = 'sandbox_app/post_list.html'


class PostDetailView(DetailView):
    model = Post
    context_object_name = 'post'
    template_name = 'sandbox_app/post_detail.html'


class PostCreateView(LoginRequiredMixin, CreateView):
    model = Post
    template_name = 'sandbox_app/post_create.html'
    login_url = '/admin/login'
    redirect_field_name = '/admin'

    def get_form_class(self):
        """
        Return different form when validation is required.
        Hidden fields are not prepopulated but appended to form by AJAX.
        """
        if self.request.method == 'POST':
            return PostAttachmentForm
        else:
            return PostForm

    def form_valid(self, form):
        form.instance.author = self.request.user

        # NOTE: Author has to be set before `form_valid()` which saves Post model instance.
        # Then, `self.object` is available in order to save attachments.
        response = super().form_valid(form)

        # Retrieve attachments not related to any post yet.
        attachments = Attachment.objects.filter(
            uid__in=form.cleaned_data['attachments'],
            post__isnull=True,
        )

        if attachments:
            self.object.attachments.set(attachments)

        return response


class PostUpdateView(AuthorRequiredMixin, LoginRequiredMixin, UpdateView):
    model = Post
    template_name = 'sandbox_app/post_update.html'
    login_url = '/admin/login'
    redirect_field_name = '/admin'

    def get_form_class(self):
        """
        Return different form when validation is required.
        Hidden fields are not prepopulated but appended to form by AJAX.
        """
        if self.request.method == 'POST':
            return PostAttachmentForm
        else:
            return PostForm

    def form_valid(self, form):
        response = super().form_valid(form)

        # Retrieve attachments not related to any post yet.
        attachments = Attachment.objects.filter(
            uid__in=form.cleaned_data['attachments'],
            post__isnull=True,
        )

        if attachments:
            attachments.update(post=self.object)

        return response


class PostDeleteView(AuthorRequiredMixin, LoginRequiredMixin, DeleteView):
    model = Post
    context_object_name = 'post'
    template_name = 'sandbox_app/post_confirm_delete.html'
    success_url = reverse_lazy('post-list')
    login_url = '/admin/login'
    redirect_field_name = '/admin'



from django.conf import settings

# The content-type header uploaded with the file (e.g. text/plain or application/pdf)
DSA_CONTENT_TYPES = getattr(settings, 'DSA_CONTENT_TYPES', ['image', 'video'])

# The allowed file extensions
DSA_FILE_EXTENSIONS = getattr(settings, 'DSA_FILE_EXTENSIONS', ['jpg', 'jpeg', 'gif', 'png'])

# The size, in bytes, of the uploaded file (default: 2MB)
DSA_MAX_UPLOAD_SIZE = getattr(settings, 'DSA_MAX_UPLOAD_SIZE', 2097152)



from os.path import splitext

from django import forms
from django.template.defaultfilters import filesizeformat
from django.utils.translation import ugettext_lazy as _

from . import settings as django_summernote_ajax_settings


class UploadAttachmentForm(forms.Form):
    files = forms.FileField(widget=forms.ClearableFileInput(
        attrs={'multiple': True}), required=False)

    def clean_files(self):
        content = self.cleaned_data['files']

        if not content:
            raise forms.ValidationError(_('Invalid file type'))

        content_type = content.content_type.split('/')[0]
        extension = splitext(content.name)[1][1:].lower()

        if extension not in django_summernote_ajax_settings.DSA_FILE_EXTENSIONS \
                or content_type not in django_summernote_ajax_settings.DSA_CONTENT_TYPES:
            raise forms.ValidationError(_('File type is not supported'))

        if content.size > django_summernote_ajax_settings.DSA_MAX_UPLOAD_SIZE:
            raise forms.ValidationError(_('Please keep filesize under %s. Current filesize %s') % (
                filesizeformat(django_summernote_ajax_settings.DSA_MAX_UPLOAD_SIZE), filesizeformat(content.size)))

        return content


class DeleteAttachmentForm(forms.Form):
    uid = forms.UUIDField()



from django.http import JsonResponse
from django.views.generic.edit import FormView

from .forms import (
    UploadAttachmentForm, DeleteAttachmentForm
)


class FileUploadView(FormView):
    """Provide a way to show and handle uploaded files in a request."""
    form_class = UploadAttachmentForm

    def upload_file(self, *args, **kwargs):
        """Abstract method must be overridden."""
        raise NotImplementedError

    def form_valid(self, form):
        """If the form is valid, return JSON file list after saving them"""
        data = self.upload_file(uploaded_files=self.request.FILES)
        return JsonResponse(data)

    def form_invalid(self, form):
        """If the form is invalid, return HTTP 400 error"""
        return JsonResponse({
            'status': 'false',
            'message': 'Bad Request'
        }, status=400)


class FileDeleteView(FormView):
    """Provide a way to show and handle files to be deleted in a request."""
    form_class = DeleteAttachmentForm

    def delete_file(self, *args, **kwargs):
        """Abstract method must be overridden."""
        raise NotImplementedError

    def form_valid(self, form):
        """If the form is valid, return JSON file list after deleting them"""
        data = self.delete_file(form=form, user=self.request.user)
        return JsonResponse(data)

    def form_invalid(self, form):
        """If the form is invalid, return HTTP 400 error"""
        return JsonResponse({
            'status': 'false',
            'message': 'Bad Request'
        }, status=400)



from setuptools import (
    setup, find_packages
)

from django_summernote_ajax import (
    VERSION, PROJECT, AUTHOR
)

setup(
    name=PROJECT,
    version=VERSION,
    description='Yet another Summernote plugin for Django',
    keywords='Django,Summernote,jQuery',
    author=AUTHOR,
    maintainer='django-summernote-ajax maintainers',
    url='http://github.com/summernote/django-summernote-ajax',
    license='MIT',
    zip_safe=False,

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    packages=find_packages(exclude=['sandbox', 'sandbox_app', 'docs', 'tests']),
    include_package_data=True,

    install_requires=[
        'Django',
    ],
    setup_requires=[
    ],
    scripts=[
    ],

    test_suite='runtests.runtests',
    tests_require=[
    ],
),



from django.apps import apps
from django.test import TestCase

from .apps import DjangoSummernoteAjaxConfig
from .widgets import SummernoteWidgetBase


class DjangoSummernoteAjaxTest(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    # apps
    def test_apps(self):
        self.assertEqual(DjangoSummernoteAjaxConfig.name, 'django_summernote_ajax')
        self.assertEqual(DjangoSummernoteAjaxConfig.verbose_name, 'Django Summernote Ajax')
        self.assertEqual(apps.get_app_config('django_summernote_ajax').name, 'django_summernote_ajax')

    # models

    # widgets
    def test_summernote_widget(self):
        widget = SummernoteWidgetBase(wrapper_class='summernote-box', options={'upload_url': '/upload-file'})
        html = widget.render(name='foo', value='bar')
        self.assertIn('summernote-widget', html)
        self.assertIn('summernote-box', html)
        self.assertIn('"upload_url": "/upload-file"', html)

    # forms

    # viewmixins

    # views



import uuid

from django.db import models
from django.utils.translation import ugettext_lazy as _


class AbstractAttachment(models.Model):
    name = models.CharField(
        max_length=255,
        null=True,
        blank=True,
        verbose_name=_('file name'),
    )

    # PK is a private identifier
    uid = models.UUIDField(
        verbose_name=_('public identifier'),
        unique=True,
        default=uuid.uuid4,
        editable=False,
    )

    file = models.FileField(
        verbose_name=_('uploaded file'),
        upload_to="attachments",
    )

    created = models.DateTimeField(
        verbose_name=_('created time'),
        auto_now_add=True,
    )

    class Meta:
        abstract = True
        verbose_name = _('attachment')
        verbose_name_plural = _('attachments')
        ordering = ['-created']

    def __str__(self):
        return self.name



import os
import uuid

from django.conf import settings
from django.core.files.storage import default_storage
from django.db import models
from django.urls import reverse
from django.utils.timezone import datetime
from django.utils.translation import ugettext_lazy as _

from django_summernote_ajax.models import AbstractAttachment


def upload_directory_path(instance, filename):
    """
    Determine directory path for uploaded files at runtime.
    """
    ext = filename.split('.')[-1]
    filename = "%s.%s" % (uuid.uuid4(), ext)
    today = datetime.now().strftime('%Y-%m-%d')
    return os.path.join('attachments', today, filename)


class Post(models.Model):
    title = models.CharField(
        verbose_name=_('title'),
        max_length=255,
    )

    body = models.TextField(
        verbose_name=_('content body'),
    )

    created = models.DateTimeField(
        verbose_name=_('created time'),
        auto_now_add=True,
    )

    author = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name=_('author'),
        related_name='posts',
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
    )

    class Meta:
        ordering = ['-created']

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('post-detail', args=[self.pk, ])


class Attachment(AbstractAttachment):
    file = models.FileField(
        verbose_name=_('uploaded file'),
        upload_to=upload_directory_path,
        storage=default_storage,
    )

    post = models.ForeignKey(
        'Post',
        verbose_name=_('post'),
        related_name='attachments',
        on_delete=models.CASCADE,
        blank=True,
        null=True,
    )
